# -*- coding: utf-8 -*-

import typing as T
import dataclasses

from ..model import CrossAccountGlobal


@dataclasses.dataclass
class S3(CrossAccountGlobal):
    @classmethod
    def new(
        cls,
        resource_id: str,
        resource_type: T.Optional[str] = None,
        sep: T.Optional[str] = None,
    ):
        return super(S3, cls).new(
            service="s3",
            resource_id=resource_id,
            partition="aws",
            resource_type=resource_type,
            sep=sep,
        )


@dataclasses.dataclass
class S3Bucket(S3):
    """
    Example: arn:aws:s3:::my-bucket
    """

    @property
    def bucket_name(self) -> str:
        return self.resource_id

    @property
    def uri(self) -> str:
        return f"s3://{self.bucket_name}"

    @classmethod
    def new(
        cls,
        bucket_name: str,
    ):
        return super(S3Bucket, cls).new(
            resource_id=bucket_name,
        )

    @classmethod
    def from_uri(cls, uri: str):
        return cls.new(uri.split("/")[2])


@dataclasses.dataclass
class S3Object(S3):
    """
    Example: arn:aws:s3:::my-bucket/folder/file.txt
    """

    @property
    def bucket(self) -> str:
        return self.resource_type

    @property
    def key(self) -> str:
        return self.resource_id

    @property
    def uri(self) -> str:
        return f"s3://{self.bucket}/{self.key}"

    @classmethod
    def new(cls, bucket: str, key: str):
        return super(S3Object, cls).new(
            resource_id=key,
            resource_type=bucket,
            sep="/",
        )

    @classmethod
    def from_uri(cls, uri: str):
        parts = uri.split("/", 3)
        return cls.new(bucket=parts[2], key=parts[3])



# -*- coding: utf-8 -*-

from .constants import AWSPartitionEnum
from .model import Arn
from .model import CrossAccountGlobal
from .model import Global
from .model import Regional
from .model import ResourceIdOnlyRegional
from .model import ColonSeparatedRegional
from .model import SlashSeparatedRegional
from .srv.iam import IamGroup
from .srv.iam import IamUser
from .srv.iam import IamRole
from .srv.iam import IamPolicy
from .srv.iam import IamInstanceProfile
from .srv.batch import BatchComputeEnvironment
from .srv.batch import BatchJobQueue
from .srv.batch import BatchJobDefinition
from .srv.batch import BatchJob
from .srv.batch import BatchSchedulingPolicy
from .srv.sagemaker_a2i import A2IHumanReviewWorkflow
from .srv.sagemaker_a2i import A2IHumanLoop
from .srv.sagemaker_a2i import A2IWorkerTaskTemplate
from .srv.cloudformation import CloudFormationStack
from .srv.cloudformation import CloudFormationChangeSet
from .srv.cloudformation import CloudFormationStackSet
from .srv.codebuild import CodeBuildProject
from .srv.codebuild import CodeBuildRun
from .srv.s3 import S3Bucket
from .srv.s3 import S3Object



# -*- coding: utf-8 -*-

import dataclasses

from ..model import SlashSeparatedRegional


@dataclasses.dataclass
class A2I(SlashSeparatedRegional):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:flow-definition/my-flow
    """
    @property
    def name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
        resource_type: str,
    ):
        return super(A2I, cls).new(
            partition="aws",
            service="sagemaker",
            region=aws_region,
            account_id=aws_account_id,
            resource_id=name,
            resource_type=resource_type,
        )


@dataclasses.dataclass
class A2IHumanReviewWorkflow(A2I):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:human-loop/a1b2
    """
    @property
    def a2i_human_review_workflow_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
    ):
        return super(A2IHumanReviewWorkflow, cls).new(
            aws_region=aws_region,
            aws_account_id=aws_account_id,
            name=name,
            resource_type="flow-definition",
        )


@dataclasses.dataclass
class A2IHumanLoop(A2I):
    """
    Example: arn:aws:sagemaker:us-east-1:111122223333:human-task-ui/my-template
    """
    @property
    def a2i_human_loop_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
    ):
        return super(A2IHumanLoop, cls).new(
            aws_region=aws_region,
            aws_account_id=aws_account_id,
            name=name,
            resource_type="human-loop",
        )


@dataclasses.dataclass
class A2IWorkerTaskTemplate(A2I):
    @property
    def a2i_worker_task_template_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
    ):
        return super(A2IWorkerTaskTemplate, cls).new(
            aws_region=aws_region,
            aws_account_id=aws_account_id,
            name=name,
            resource_type="human-task-ui",
        )



# -*- coding: utf-8 -*-


def test():
    from aws_arns import api

    _ = api.Arn
    _ = api.AWSPartitionEnum
    _ = api.Arn
    _ = api.CrossAccountGlobal
    _ = api.Global
    _ = api.Regional
    _ = api.ResourceIdOnlyRegional
    _ = api.ColonSeparatedRegional
    _ = api.SlashSeparatedRegional
    _ = api.IamGroup
    _ = api.IamUser
    _ = api.IamRole
    _ = api.IamPolicy
    _ = api.IamInstanceProfile
    _ = api.BatchComputeEnvironment
    _ = api.BatchJobQueue
    _ = api.BatchJobDefinition
    _ = api.BatchJob
    _ = api.BatchSchedulingPolicy
    _ = api.A2IHumanReviewWorkflow
    _ = api.A2IHumanLoop
    _ = api.A2IWorkerTaskTemplate
    _ = api.CloudFormationStack
    _ = api.CloudFormationChangeSet
    _ = api.CloudFormationStackSet
    _ = api.CodeBuildProject
    _ = api.CodeBuildRun
    _ = api.S3Bucket
    _ = api.S3Object


if __name__ == "__main__":
    from aws_arns.tests.helper import run_cov_test

    run_cov_test(__file__, "aws_arns.api", preview=False)



# -*- coding: utf-8 -*-

import dataclasses

from ..model import SlashSeparatedRegional


@dataclasses.dataclass
class Batch(SlashSeparatedRegional):
    @property
    def name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
        resource_type: str,
    ):
        return super(Batch, cls).new(
            partition="aws",
            service="batch",
            region=aws_region,
            account_id=aws_account_id,
            resource_id=name,
            resource_type=resource_type,
        )


@dataclasses.dataclass
class BatchComputeEnvironment(Batch):
    """
    Example: arn:aws:batch:us-east-1:111122223333:compute-environment/my-ce
    """
    @property
    def batch_compute_environment_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
    ):
        return super(BatchComputeEnvironment, cls).new(
            aws_region=aws_region,
            aws_account_id=aws_account_id,
            name=name,
            resource_type="compute-environment",
        )


@dataclasses.dataclass
class BatchJobQueue(Batch):
    """
    Example: arn:aws:batch:us-east-1:111122223333:job-queue/my-queue
    """
    @property
    def batch_job_queue_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
    ):
        return super(BatchJobQueue, cls).new(
            aws_region=aws_region,
            aws_account_id=aws_account_id,
            name=name,
            resource_type="job-queue",
        )


@dataclasses.dataclass
class BatchJobDefinition(Batch):
    """
    Example: arn:aws:batch:us-east-1:111122223333:job-definition/my-job-def:1
    """
    @property
    def batch_job_definition_fullname(self) -> str:
        return self.resource_id

    @property
    def batch_job_definition_name(self) -> str:
        return self.resource_id.split(":")[0]

    @property
    def batch_job_definition_revision(self) -> int:
        return int(self.resource_id.split(":")[1])

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
        revision: int,
    ):
        return super(BatchJobDefinition, cls).new(
            aws_region=aws_region,
            aws_account_id=aws_account_id,
            name=f"{name}:{revision}",
            resource_type="job-definition",
        )


@dataclasses.dataclass
class BatchJob(Batch):
    """
    Example: arn:aws:batch:us-east-1:111122223333:job/a974ee84-1da8-40bf-bca9-ef4253fac3c6
    """
    @property
    def batch_job_id(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        job_id: str,
    ):
        return super(BatchJob, cls).new(
            aws_region=aws_region,
            aws_account_id=aws_account_id,
            name=job_id,
            resource_type="job",
        )


@dataclasses.dataclass
class BatchSchedulingPolicy(Batch):
    """
    Example: arn:aws:batch:us-east-1:111122223333:scheduling-policy/my-policy
    """
    @property
    def batch_scheduling_policy_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
    ):
        return super(BatchSchedulingPolicy, cls).new(
            aws_region=aws_region,
            aws_account_id=aws_account_id,
            name=name,
            resource_type="scheduling-policy",
        )



# -*- coding: utf-8 -*-

import pytest
import itertools
from aws_arns.model import (
    Arn,
    CrossAccountGlobal,
    Global,
    Regional,
    ResourceIdOnlyRegional,
    ColonSeparatedRegional,
    SlashSeparatedRegional,
)

cloudformation = [
    "arn:aws:cloudformation:us-east-1:111122223333:stack/stack-name/8e6db190-bd6a-11ed-b80d-12cc1b6777a1",
    "arn:aws:cloudformation:us-east-1:111122223333:changeSet/stack-name-2023-03-08-04-42-38-564/5be009b1-f057-44b0-a5e3-cd9bf4a24b9e",
    "arn:aws:cloudformation:us-east-1:111122223333:stackset/stack-name:08af3d48-ec8e-45ad-b109-363d27fcf851",
]

kinesis = [
    "arn:aws:kinesisvideo:us-east-1:111122223333:stream/kinesis-stream-name/111122223333",
]

cloudwatch_logs = [
    "arn:aws:logs:us-east-1:111122223333:log-group:/aws/lambda/my-func:*",
    "arn:aws:logs:us-east-1:111122223333:log-group:my-log-group*:log-stream:my-log-stream*",
]

macie = [
    "arn:aws:macie:us-east-1:111122223333:trigger/example0954663fda0f652e304dcc21323508db/alert/example09214d3e70fb6092cc93cee96dbc4de6",
]

s3 = [
    "arn:aws:s3:::my-bucket",
    "arn:aws:s3:::my-bucket/cloudformation/upload/10f3db7bcfa62c69e5a71fef595fac84.json",
]

ec2 = [
    "arn:aws:ec2:us-east-1:111122223333:instance/*",
]

lambda_func = [
    "arn:aws:lambda:us-east-1:111122223333:function:my-func",
    "arn:aws:lambda:us-east-1:111122223333:function:my-func:LIVE",
    "arn:aws:lambda:us-east-1:111122223333:function:my-func:1",
]

apigateway = [
    "arn:aws:apigateway:us-east-1::7540694639748281fa84fabba58e57c0:/test/mydemoresource/*",
]

sns = [
    "arn:aws:sns:*:111122223333:my_topic",
]

secretmanager = [
    "arn:aws:secretsmanager:us-east-1:111122223333:secret:MyFolder/MySecret-a1b2c3",
]

batch = [
    "arn:aws:batch:us-east-1:111122223333:job-definition/my-job:1",
]

arns = list(
    itertools.chain(
        cloudformation,
        kinesis,
        cloudwatch_logs,
        macie,
        s3,
        ec2,
        lambda_func,
        apigateway,
        sns,
        secretmanager,
        batch,
    )
)


def test_from_and_to():
    for arn_str in arns:
        arn = Arn.from_arn(arn_str)
        assert arn.to_arn() == arn_str


def test_error():
    with pytest.raises(ValueError):
        Arn.from_arn("hello")


class TestCrossAccountGlobal:
    def test(self):
        s3_bucket = CrossAccountGlobal.new(
            service="s3",
            resource_id="my-bucket",
        )
        arn = "arn:aws:s3:::my-bucket"
        assert s3_bucket.to_arn() == arn

        s3_bucket_1 = CrossAccountGlobal.from_arn(arn)
        for thing in [s3_bucket, s3_bucket_1]:
            assert thing.service == "s3"
            assert thing.region == None
            assert thing.account_id == None
            assert thing.resource_type == None
            assert thing.resource_id == "my-bucket"
            assert thing.sep == None


class TestGlobal:
    def test(self):
        iam_role = Global.new(
            service="iam",
            resource_id="my-role",
            account_id="111122223333",
        )
        arn = "arn:aws:iam::111122223333:my-role"
        assert iam_role.to_arn() == arn

        iam_role_1 = Global.from_arn(arn)
        for thing in [iam_role, iam_role_1]:
            assert thing.service == "iam"
            assert thing.region == None
            assert thing.account_id == "111122223333"
            assert thing.resource_type == None
            assert thing.resource_id == "my-role"
            assert thing.sep == None


class TestRegional:
    def test(self):
        lbd_func = Regional.new(
            service="lambda",
            resource_id="my-func",
            region="us-east-1",
            account_id="111122223333",
            resource_type="function",
            sep=":",
        )
        assert lbd_func.aws_region == "us-east-1"
        assert lbd_func.aws_account_id == "111122223333"
        arn = "arn:aws:lambda:us-east-1:111122223333:function:my-func"
        assert lbd_func.to_arn() == arn

        lbd_func_1 = Regional.from_arn(arn)
        for thing in [lbd_func, lbd_func_1]:
            assert thing.service == "lambda"
            assert thing.region == "us-east-1"
            assert thing.account_id == "111122223333"
            assert thing.resource_type == "function"
            assert thing.resource_id == "my-func"
            assert thing.sep == ":"


class TestResourceIdOnlyRegional:
    def test(self):
        sns_topic = ResourceIdOnlyRegional.new(
            service="sns",
            resource_id="my-topic",
            region="us-east-1",
            account_id="111122223333",
        )
        arn = "arn:aws:sns:us-east-1:111122223333:my-topic"
        assert sns_topic.to_arn() == arn

        sns_topic_1 = ResourceIdOnlyRegional.from_arn(arn)
        for thing in [sns_topic, sns_topic_1]:
            assert thing.service == "sns"
            assert thing.region == "us-east-1"
            assert thing.account_id == "111122223333"
            assert thing.resource_type == None
            assert thing.resource_id == "my-topic"
            assert thing.sep == None


class TestColonSeparatedRegional:
    def test(self):
        lbd_func = ColonSeparatedRegional.new(
            service="lambda",
            resource_id="my-func",
            region="us-east-1",
            account_id="111122223333",
            resource_type="function",
        )
        arn = "arn:aws:lambda:us-east-1:111122223333:function:my-func"
        assert lbd_func.to_arn() == arn

        lbd_func_1 = ColonSeparatedRegional.from_arn(arn)
        for thing in [lbd_func, lbd_func_1]:
            assert thing.service == "lambda"
            assert thing.region == "us-east-1"
            assert thing.account_id == "111122223333"
            assert thing.resource_type == "function"
            assert thing.resource_id == "my-func"
            assert thing.sep == ":"


class TestSlashSeparatedRegional:
    def test(self):
        cf_stack = SlashSeparatedRegional.new(
            service="cloudformation",
            resource_id="my-stack",
            region="us-east-1",
            account_id="111122223333",
            resource_type="stack",
        )
        arn = "arn:aws:cloudformation:us-east-1:111122223333:stack/my-stack"
        assert cf_stack.to_arn() == arn

        cf_stack_1 = SlashSeparatedRegional.from_arn(arn)
        for thing in [cf_stack, cf_stack_1]:
            assert thing.service == "cloudformation"
            assert thing.region == "us-east-1"
            assert thing.account_id == "111122223333"
            assert thing.resource_type == "stack"
            assert thing.resource_id == "my-stack"
            assert thing.sep == "/"


if __name__ == "__main__":
    from aws_arns.tests.helper import run_cov_test

    run_cov_test(__file__, "aws_arns.model", preview=False)



# -*- coding: utf-8 -*-

import dataclasses

from ..model import SlashSeparatedRegional


@dataclasses.dataclass
class CodeBuild(SlashSeparatedRegional):
    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        fullname: str,
        resource_type: str,
    ):
        return super(CodeBuild, cls).new(
            partition="aws",
            service="codebuild",
            region=aws_region,
            account_id=aws_account_id,
            resource_id=fullname,
            resource_type=resource_type,
        )


@dataclasses.dataclass
class CodeBuildProject(CodeBuild):
    """
    Example: arn:aws:codebuild:us-east-1:111122223333:project/my-project
    """

    @property
    def codebuild_project_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        name: str,
    ):
        return super(CodeBuildProject, cls).new(
            aws_region=aws_region,
            aws_account_id=aws_account_id,
            fullname=name,
            resource_type="project",
        )


@dataclasses.dataclass
class CodeBuildRun(CodeBuild):
    """
    Example: arn:aws:codebuild:us-east-1:111122223333:build/my-project:a1b2c3d4
    """

    @property
    def codebuild_run_fullname(self) -> str:
        return self.resource_id

    @property
    def codebuild_project_name(self) -> str:
        return self.resource_id.split(":")[0]

    @property
    def codebuild_run_id(self) -> str:
        return self.resource_id.split(":")[-1]

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        fullname: str,
    ):
        return super(CodeBuildRun, cls).new(
            aws_region=aws_region,
            aws_account_id=aws_account_id,
            fullname=fullname,
            resource_type="build",
        )



# -*- coding: utf-8 -*-

"""
This is for internal use, don't import
"""

import dataclasses
from ..model import (
    CrossAccountGlobal,
    Global,
    Regional,
    ResourceIdOnlyRegional,
    ColonSeparatedRegional,
    SlashSeparatedRegional,
)


@dataclasses.dataclass
class S3(CrossAccountGlobal):
    pass


# --- ResourceIdOnlyRegional
@dataclasses.dataclass
class SQS(ResourceIdOnlyRegional):
    pass


@dataclasses.dataclass
class SNS(ResourceIdOnlyRegional):
    pass


@dataclasses.dataclass
class CodeCommit(ResourceIdOnlyRegional):
    pass


@dataclasses.dataclass
class CodePipeline(ResourceIdOnlyRegional):
    pass


# --- ColonSeparatedRegional
@dataclasses.dataclass
class Lambda(ColonSeparatedRegional):
    pass


@dataclasses.dataclass
class Sfn(ColonSeparatedRegional):
    pass


@dataclasses.dataclass
class SecretManager(ColonSeparatedRegional):
    pass


# --- SlashSeparatedRegional
@dataclasses.dataclass
class CloudFormation(SlashSeparatedRegional):
    pass


@dataclasses.dataclass
class CodeBuild(SlashSeparatedRegional):
    pass


@dataclasses.dataclass
class ECS(SlashSeparatedRegional):
    pass


@dataclasses.dataclass
class Dynamodb(SlashSeparatedRegional):
    pass


@dataclasses.dataclass
class Glue(SlashSeparatedRegional):
    pass


@dataclasses.dataclass
class KMS(SlashSeparatedRegional):
    pass


@dataclasses.dataclass
class SageMaker(SlashSeparatedRegional):
    pass


@dataclasses.dataclass
class SSMParameterStore(SlashSeparatedRegional):
    pass



# -*- coding: utf-8 -*-

import dataclasses

from ..model import SlashSeparatedRegional


@dataclasses.dataclass
class CloudFormation(SlashSeparatedRegional):
    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        fullname: str,
        resource_type: str,
    ):
        return super(CloudFormation, cls).new(
            partition="aws",
            service="cloudformation",
            region=aws_region,
            account_id=aws_account_id,
            resource_id=fullname,
            resource_type=resource_type,
        )


@dataclasses.dataclass
class CloudFormationStack(CloudFormation):
    """
    Example: arn:aws:cloudformation:us-east-1:111122223333:stack/my-stack/a1b2c3d4
    """

    @property
    def stack_name(self) -> str:
        return self.resource_id.split("/")[0]

    @property
    def stack_id(self) -> str:
        return self.resource_id.split("/")[1]

    @property
    def stack_fullname(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        fullname: str,
    ):
        return super(CloudFormationStack, cls).new(
            aws_region=aws_region,
            aws_account_id=aws_account_id,
            fullname=fullname,
            resource_type="stack",
        )


@dataclasses.dataclass
class CloudFormationChangeSet(CloudFormation):
    """
    Example: arn:aws:cloudformation:us-east-1:111122223333:changeSet/my-change-set/a1b2c3d4
    """

    @property
    def changeset_fullname(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        fullname: str,
    ):
        return super(CloudFormationChangeSet, cls).new(
            aws_region=aws_region,
            aws_account_id=aws_account_id,
            fullname=fullname,
            resource_type="changeSet",
        )


@dataclasses.dataclass
class CloudFormationStackSet(CloudFormation):
    """
    Example: arn:aws:cloudformation:us-east-1:111122223333:stackset/my-stackset:a1b2c3d4
    """

    @property
    def stackset_name(self) -> str:
        return self.resource_id.split(":")[0]

    @property
    def stackset_id(self) -> str:
        return self.resource_id.split(":")[1]

    @property
    def stackset_fullname(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        aws_region: str,
        fullname: str,
    ):
        return super(CloudFormationStackSet, cls).new(
            aws_region=aws_region,
            aws_account_id=aws_account_id,
            fullname=fullname,
            resource_type="stackset",
        )



# -*- coding: utf-8 -*-

import dataclasses
from ..model import Global


@dataclasses.dataclass
class Iam(Global):
    @classmethod
    def new(
        cls,
        aws_account_id: str,
        resource_type: str,
        name: str,
    ):
        return super(Iam, cls).new(
            partition="aws",
            service="iam",
            account_id=aws_account_id,
            resource_id=name,
            resource_type=resource_type,
            sep="/",
        )

    @property
    def short_name(self) -> str:
        return self.resource_id.split("/")[-1]


@dataclasses.dataclass
class IamGroup(Iam):
    """
    Example: arn:aws:iam::111122223333:group/Admin
    """
    @property
    def iam_group_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        name: str,
    ) -> "IamGroup":
        return super(IamGroup, cls).new(
            aws_account_id=aws_account_id,
            resource_type="group",
            name=name,
        )


@dataclasses.dataclass
class IamUser(Iam):
    """
    Example: arn:aws:iam::111122223333:user/alice
    """
    @property
    def iam_user_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        name: str,
    ) -> "IamUser":
        return super(IamUser, cls).new(
            aws_account_id=aws_account_id,
            resource_type="user",
            name=name,
        )


@dataclasses.dataclass
class IamRole(Iam):
    """
    Example: arn:aws:iam::111122223333:role/aws-service-role/batch.amazonaws.com/AWSServiceRoleForBatch
    """
    @property
    def iam_role_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        name: str,
    ) -> "IamRole":
        return super(IamRole, cls).new(
            aws_account_id=aws_account_id,
            resource_type="role",
            name=name,
        )

    def is_service_role(self) -> bool:
        return self.iam_role_name.startswith("aws-service-role")


@dataclasses.dataclass
class IamPolicy(Iam):
    """
    Example: arn:aws:iam::111122223333:policy/service-role/codebuild-policy
    """
    @property
    def iam_policy_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        name: str,
    ) -> "IamPolicy":
        return super(IamPolicy, cls).new(
            aws_account_id=aws_account_id,
            resource_type="policy",
            name=name,
        )


@dataclasses.dataclass
class IamInstanceProfile(Iam):
    """
    Example: arn:aws:iam::111122223333:instance-profile/cloud9/AWSCloud9SSMInstanceProfile
    """
    @property
    def iam_instance_profile_name(self) -> str:
        return self.resource_id

    @classmethod
    def new(
        cls,
        aws_account_id: str,
        name: str,
    ) -> "IamInstanceProfile":
        return super(IamInstanceProfile, cls).new(
            aws_account_id=aws_account_id,
            resource_type="instance-profile",
            name=name,
        )



# -*- coding: utf-8 -*-

import typing as T
import dataclasses

from . import compat


def _handle_empty_str(s: str) -> T.Optional[str]:
    if s:
        return s
    else:
        return None


def _handle_none(s: T.Optional[str]) -> str:
    if s is None:
        return ""
    else:
        return s


@dataclasses.dataclass
class Arn:
    """
    Amazon Resource Names (ARNs) data model. is a unique identifier for AWS resources.

    ARN format::

        - format: arn:partition:service:region:account-id:resource-id
        - example: arn:aws:sqs:us-east-1:111122223333:my-queue
        - format: arn:partition:service:region:account-id:resource-type/resource-id
        - example: arn:aws:iam::111122223333:role/aws-service-role/batch.amazonaws.com/AWSServiceRoleForBatch
        - format: arn:partition:service:region:account-id:resource-type:resource-id
        - example: arn:aws:batch:us-east-1:111122223333:job-definition/my-job-def:1

    Reference:

    - Amazon Resource Names (ARNs): https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html
    """

    partition: str = dataclasses.field()
    service: str = dataclasses.field()
    region: T.Optional[str] = dataclasses.field()
    account_id: T.Optional[str] = dataclasses.field()
    resource_type: T.Optional[str] = dataclasses.field()
    resource_id: str = dataclasses.field()
    sep: T.Optional[str] = dataclasses.field()

    @classmethod
    def new(
        cls,
        service: str,
        resource_id: str,
        partition: str = "aws",
        region: T.Optional[str] = None,
        account_id: T.Optional[str] = None,
        resource_type: T.Optional[str] = None,
        sep: T.Optional[str] = None,
    ):
        """
        new 这个构造器的目的是为了方便创建一个 Arn 对象.
        """
        return cls(
            service=service,
            resource_id=resource_id,
            partition=partition,
            region=region,
            account_id=account_id,
            resource_type=resource_type,
            sep=sep,
        )

    @classmethod
    def from_arn(cls, arn: str) -> "Arn":
        """
        parse arn string into Arn object.
        """
        if not arn.startswith("arn:"):
            raise ValueError(f"Invalid ARN: {arn!r}")

        _, partition, service, region, account_id, resource = arn.split(":", 5)

        if "/" in resource:
            sep = "/"
            resource_type, resource_id = resource.split("/", 1)
        elif ":" in resource:
            sep = ":"
            resource_type, resource_id = resource.split(":", 1)
        else:
            sep = None
            resource_type, resource_id = None, resource

        return cls(
            partition=partition,
            service=service,
            region=_handle_empty_str(region),
            account_id=_handle_empty_str(account_id),
            resource_id=resource_id,
            resource_type=resource_type,
            sep=sep,
        )

    def to_arn(self) -> str:
        """
        convert Arn object into arn string.
        """
        if self.sep:
            resource = f"{self.resource_type}{self.sep}{self.resource_id}"
        else:
            resource = self.resource_id
        return f"arn:{self.partition}:{self.service}:{_handle_none(self.region)}:{_handle_none(self.account_id)}:{resource}"

    @property
    def aws_account_id(self) -> T.Optional[str]:
        return self.account_id

    @property
    def aws_region(self) -> T.Optional[str]:
        return self.region


@dataclasses.dataclass
class CrossAccountGlobal(Arn):
    """
    No account, no region. Example:

    - AWS S3
    """

    @classmethod
    def new(
        cls,
        service: str,
        resource_id: str,
        partition: str = "aws",
        resource_type: T.Optional[str] = None,
        sep: T.Optional[str] = None,
    ):
        return super(CrossAccountGlobal, cls).new(
            partition=partition,
            service=service,
            region=None,
            account_id=None,
            resource_id=resource_id,
            resource_type=resource_type,
            sep=sep,
        )


@dataclasses.dataclass
class Global(Arn):
    """
    No region. Example:

    - AWS IAM
    - AWS Route53
    """

    @classmethod
    def new(
        cls,
        service: str,
        resource_id: str,
        account_id: str,
        partition: str = "aws",
        resource_type: T.Optional[str] = None,
        sep: T.Optional[str] = None,
    ):
        return super(Global, cls).new(
            partition=partition,
            service=service,
            region=None,
            account_id=account_id,
            resource_id=resource_id,
            resource_type=resource_type,
            sep=sep,
        )


@dataclasses.dataclass
class Regional(Arn):
    """
    Normal regional resources. Example:

    - AWS SQS
    - AWS Lambda
    """

    @classmethod
    def new(
        cls,
        service: str,
        resource_id: str,
        region: str,
        account_id: str,
        partition: str = "aws",
        resource_type: T.Optional[str] = None,
        sep: T.Optional[str] = None,
    ):
        return super(Regional, cls).new(
            partition=partition,
            service=service,
            region=region,
            account_id=account_id,
            resource_id=resource_id,
            resource_type=resource_type,
            sep=sep,
        )


@dataclasses.dataclass
class ResourceIdOnlyRegional(Arn):
    """
    Only one resource type in this service. Example:

    - AWS SQS
    - AWS SNS
    """

    @classmethod
    def new(
        cls,
        service: str,
        resource_id: str,
        region: str,
        account_id: str,
        partition: str = "aws",
    ):
        return super(ResourceIdOnlyRegional, cls).new(
            partition=partition,
            service=service,
            region=region,
            account_id=account_id,
            resource_id=resource_id,
            resource_type=None,
            sep=None,
        )


@dataclasses.dataclass
class ColonSeparatedRegional(Arn):
    """
    Example:

    - AWS Lambda
    """

    @classmethod
    def new(
        cls,
        service: str,
        resource_id: str,
        region: str,
        account_id: str,
        partition: str = "aws",
        resource_type: T.Optional[str] = None,
    ):
        return super(ColonSeparatedRegional, cls).new(
            partition=partition,
            service=service,
            region=region,
            account_id=account_id,
            resource_id=resource_id,
            resource_type=resource_type,
            sep=":",
        )


@dataclasses.dataclass
class SlashSeparatedRegional(Arn):
    """
    Example:

    - AWS CloudFormation
    """

    @classmethod
    def new(
        cls,
        service: str,
        resource_id: str,
        region: str,
        account_id: str,
        partition: str = "aws",
        resource_type: T.Optional[str] = None,
    ):
        return super(SlashSeparatedRegional, cls).new(
            partition=partition,
            service=service,
            region=region,
            account_id=account_id,
            resource_id=resource_id,
            resource_type=resource_type,
            sep="/",
        )
